//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA logfile parser
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/logfilestatistics.h,v 1.5 2003/01/24 15:47:41 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef YASAGUI_LOGFILESTATISTICS_PARSER_INCLUDE
#define	YASAGUI_LOGFILESTATISTICS_PARSER_INCLUDE

#include <qstring.h>
#include <yasagui/yasagui.h>
#include "datatypes.h"


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YLogFileTask;
class YLogFileMutex;
class YProjectLogFile;
class YCPULogFile;
class YEnvironmentLogFileSet;


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YStatisticBaseEntry
{
public: virtual		~YStatisticBaseEntry()					{												}
	//! Returns true if the content is valid
	virtual bool	IsValid() const	=0;
	//! Set the content to invalid
	virtual void	Invalid() =0;
	//! Reset the content to its initial state
	virtual void	Reset() =0;
	//! Returns the visible string of the content
	virtual QString	GetVisibleString() const =0;
	//! Returns a string used to sorting more than one item of this type
	virtual QString	GetKey() const =0;
	//! Return the visible string used for contents which are unknown
	virtual void	IncContent()							{												}
	//! Return the visible string used for contents which are unknown
	static QString	GetUnknownString()						{	return MSGP_NOT_APPLICABLE;					}
	//! Return the visible string used for contents which are invalid
	static QString	GetInvalidString()						{	return MSGP_NOT_APPLICABLE;					}
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
template<class T>
class YStatisticEntry : public YStatisticBaseEntry
{
	//! contains the state of the content during construction. Needed if Reset() is called.
	bool			initvalid;
	//! contains the value given during construction. Needed if Reset() is called.
	T				initcontent;
	//! Is this statistic entry set correctly ?
	bool			valid;
	//! content value, represents the current valid value (if valid=true)
	T				content;
protected:
	//! Returns the visible string of the content
	virtual QString	_GetVisibleString() const =0;
	//! Returns a string used to sorting more than one item of this type
	virtual QString	_GetKey() const							{	return GetVisibleString();					}
public:				YStatisticEntry() : initvalid(false), valid(false)										{ }
					YStatisticEntry(const T &t) : initvalid(true), initcontent(t), valid(true), content(t)	{ }
	//! Returns true if the content is valid
	virtual bool	IsValid() const							{	return valid;								}
	//! Set the content to invalid
	virtual void	Invalid()								{	valid=false;								}
	//! Reset the content to its initial state
	virtual void	Reset()									{	content=initcontent;
																valid=initvalid;							}
	//! Set a valid content
	const T &		SetContent(const T &t)					{	content=t;
																valid=true;
																return content;								}
	//! Get a content. This function may only be called if the content is valid, see YStatisticEntry::IsValid()
	const T &		GetContent() const						{	return content;								}
	//! Returns the visible string of the content
	virtual QString	GetVisibleString() const				{	if (valid)
																	return _GetVisibleString();
																return GetInvalidString();					}
	virtual QString	GetKey() const							{	if (valid)
																	return _GetKey();
																//return QString::null;						}
																return QString(LOWEST_STRING);				}
	virtual const T & operator = (const T &t)				{	return SetContent(t);						}
	/*	virtual const T & operator ++(void)							{	if ( IsValid() )
																	SetContent( content+1 );
																return content;								}
	virtual const T & operator ++(int i)						{	if ( IsValid() )
																	SetContent( content+i );
																return content;								}
*/
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YYasaTimeStatisticEntry : public YStatisticEntry<YASA_TIME>
{public:			YYasaTimeStatisticEntry() : YStatisticEntry<YASA_TIME>()								{ }
					YYasaTimeStatisticEntry(const YASA_TIME &t) : YStatisticEntry<YASA_TIME>(t)				{ }
protected:
	//! This function returns the visible content string of the statistic entry.
	//! It is allowed to call the function only if the content is valid.
	virtual QString	_GetVisibleString() const				{	return YTIME::GetVisibleContentString( GetContent() ); }
	virtual QString	_GetKey() const;
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This class contains strings. Default value is 0
class YZeroYasaTimeStatisticEntry : public YYasaTimeStatisticEntry
{public:			YZeroYasaTimeStatisticEntry() : YYasaTimeStatisticEntry(0)									{ }
};
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This class contains strings. Default value is YASA_TIME_INFINITY
class YInfinityYasaTimeStatisticEntry : public YYasaTimeStatisticEntry
{public:			YInfinityYasaTimeStatisticEntry() : YYasaTimeStatisticEntry(YASA_TIME_INFINITY)			{ }
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YBoolStatisticEntry : public YStatisticEntry<bool>
{public:			YBoolStatisticEntry() : YStatisticEntry<bool>()											{ }
					YBoolStatisticEntry(const bool &b) : YStatisticEntry<bool>(b)							{ }
protected:
	//! This function returns the visible content string of the statistic entry.
	//! It is allowed to call the function only if the content is valid.
	virtual QString	_GetVisibleString() const				{	if ( GetContent() )
																	return MSGP_ERROR_YES;
																else
																return MSGP_ERROR_NO;						}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This class contains strings. Default value is QString::null
class YStringStatisticEntry : public YStatisticEntry<QString>
{public:			YStringStatisticEntry(const QString &s=QString::null) : YStatisticEntry<QString>(s)		{ }
protected:
	//! This function returns the visible content string of the statistic entry.
	//! It is allowed to call the function only if the content is valid.
	virtual QString	_GetVisibleString() const				{	return GetContent();						}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This class contains integer variables. Default value is 0
class YIntStatisticEntry : public YStatisticEntry<int>
{public:			YIntStatisticEntry(const int &i=0) : YStatisticEntry<int>(i)							{ }
protected:
	//! This function returns the visible content string of the statistic entry.
	//! It is allowed to call the function only if the content is valid.
	virtual QString	_GetVisibleString() const				{	return QString::number( GetContent() );		}
	//! Increment integer
	virtual void	IncContent()							{	if ( IsValid() )
																	SetContent( GetContent()+1 );			}
	virtual QString	_GetKey() const;
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This class contains priority variables. Default value is 0
class YPriorityStatisticEntry : public YStatisticEntry<int>
{public:			YPriorityStatisticEntry(const int &i=0) : YStatisticEntry<int>(i)						{ }
protected:
	//! This function returns the visible content string of the statistic entry.
	//! It is allowed to call the function only if the content is valid.
	virtual QString	_GetVisibleString() const				{	return QString::number( GetContent() );		}
	virtual QString	_GetKey() const;
};


#define PERCENTTYPE	double
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This class contains integer variables. Default value is 0
class YPercentStatisticEntry : public YStatisticEntry<PERCENTTYPE>
{public:			YPercentStatisticEntry() : YStatisticEntry<PERCENTTYPE>()								{ }
					YPercentStatisticEntry(const PERCENTTYPE &i) : YStatisticEntry<PERCENTTYPE>(i)			{ }
protected:
	//! This function returns the visible content string of the statistic entry.
	//! It is allowed to call the function only if the content is valid.
	virtual QString	_GetVisibleString() const								{	return QString("%1 %").arg( ((float)((int)(GetContent()*100)))/100 );	}
public:
	void			CalcContent( YASA_TIME G, YASA_TIME W)					{	if ( G )
																					SetContent( (PERCENTTYPE)( W*100) / G );	}
	void			CalcContent( int G, int W)								{	if ( G )
																					SetContent( (PERCENTTYPE)( W*100) / G );	}
	void			CalcContent( double G, double W)						{	if ( G )
																					SetContent( (PERCENTTYPE)( W*100) / G );	}
	virtual QString	_GetKey() const;
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This class contains priority variables. Default value is 0
class YVersionStatisticEntry : public YStatisticEntry<double>
{public:			YVersionStatisticEntry() : YStatisticEntry<double>()									{ }
					YVersionStatisticEntry(const double &i) : YStatisticEntry<double>(i)					{ }
protected:
	//! This function returns the visible content string of the statistic entry.
	//! It is allowed to call the function only if the content is valid.
	virtual QString	_GetVisibleString() const				{	return QString::number( GetContent() );		}
	virtual QString	_GetKey() const;
};








//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This class contains the global statistic entries of one project
class YLogFileStatistics
{protected:
	QArray<YStatisticBaseEntry*>properties;
	void					CheckProperties()
							{
								for ( int count=0; count<(int)properties.count(); count++ )
								{	if ( properties[count]==0 )
										qWarning("YLogFileStatistics::CheckProperties() failed!");
								}
							}
public:						YLogFileStatistics(int number) : properties(number)							{ }
	virtual					~YLogFileStatistics()														{ }
	int						GetMaxProperties() const
							{
								return properties.count();
							}
	virtual void			ResetStatistics()
							{
								for ( int count=0; count<(int)properties.count(); count++ )
								{	if ( properties[count]!=0 )
										properties[count]->Reset();
								}
							}
	virtual YStatisticBaseEntry& GetStatisticProperty(int number) const		{	return *properties[number];	}
	virtual QStringList		GetPropertyNames() const=0;
	void					CalculateTimingValues(YEnvironmentLogFileSet &envlogfile, QList<YASA_TIME> &list, int firstproperty, int propertycount);
	bool					IsValid(int number) const
							{
								if ( number < (int)properties.count() )
									return properties[number]->IsValid();
								return false;
							}
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This class contains the task statistic entries of one task
class YTaskLogFileStatistics : public YLogFileStatistics
{	YLogFileTask *			task;
	QList<YASA_TIME>		computationtimes;
	QList<YASA_TIME>		jittertimes;
	QList<YASA_TIME>		reactiontimes;
public:	enum TASK_PROPERTIES {
							ID=0,						// Id of task
							PERIOD,						// period of thread
							DEADLINE,					// deadline of thread
							ACTIVATIONS,				// number of activations in scheduling period
							STARTS,						// number of starting periods in scheduling period
							THREADS_SUSPENDED,			// how many times the thread was suspended
							PREEMPTIONS,				// number of preemptions
							CPU_LOSSES,					// number of cpu losses
							EXCLUSIONS,					// number of exclusion (ELLF)
							MATCHED_DEADLINES,			// number of matched deadlines
							MISSED_DEADLINES,			// number of missed deadlines
							DETECTED_DEADLINE_MISSES,	// number of discovered deadline misses
							DEADLINE_TOLERANCE_STARTS,	// number of tolerance deadline starts
							DEADLINE_TOLERANCE_MISSES,	// number of tolerance deadline misses
							//DETECTED_TOLERANCE_MISSES,// number of detected tolerance deadline misses
							SYNCHRONIZATION_OFFSET,		// synchronization offset
	/* do not change the order --> */
							MIN_COMPUTATION_TIME,		// min computation time
							MAX_COMPUTATION_TIME,		// max computation time
							AVERAGE_COMPUTATION_TIME,	// average computation time
							COMPUTATION_TIME,			// computation time of this task in the scheduling interval
							UTILIZATION,				// utilization in scheduling period
							MIN_REACTION_TIME,			// min. time until thread finished since activation
							MAX_REACTION_TIME,			// max. time until thread finished since activation
							AVERAGE_REACTION_TIME,		// average time until thread finished since activation
							MIN_JITTER,
							MAX_JITTER,
							AVERAGE_JITTER,
							//JITTER_PERCENT,
	/* <-- do not change the order */
							SKIPPED_THREADS,			// skipped threads
							EMERGENCY_THREADS,			// how many times an emergency thread was started
							LOCKED_MUTEXES,				// number of locked mutexes
							UNLOCKED_MUTEXES,			// number of unlocked mutexes
							RELOCKED_MUTEXES,			// locked a mutex twice or more ?
							BLOCKED_MUTEXES,
							MAX_PRIORITY_CEILING,		// max. priority ceiling
							NUMBER_OF_PRIORITY_CHANGES,// number of priority changes (How many times did a task change its priority)
							MAX_PROPERTIES
						};

						YTaskLogFileStatistics() : YLogFileStatistics(MAX_PROPERTIES), task(0)
						{
							computationtimes.setAutoDelete(true);
							jittertimes.setAutoDelete(true);
							reactiontimes.setAutoDelete(true);
							properties[ID]							= new YIntStatisticEntry;			// id of task
							properties[PERIOD]						= new YInfinityYasaTimeStatisticEntry;	// period of thread
							properties[DEADLINE]					= new YInfinityYasaTimeStatisticEntry;	// deadline of thread
							properties[ACTIVATIONS]					= new YIntStatisticEntry;			// number of activations in scheduling period
							properties[STARTS]						= new YIntStatisticEntry;			// number of starting periods in scheduling period
							properties[THREADS_SUSPENDED]			= new YIntStatisticEntry;			// how many times the thread was suspended
							properties[PREEMPTIONS]					= new YIntStatisticEntry;			// number of preemptions
							properties[EXCLUSIONS]					= new YIntStatisticEntry;			// number of exclusion (ELLF)
							properties[CPU_LOSSES]					= new YIntStatisticEntry;			// number of exclusion (ELLF)
							properties[MATCHED_DEADLINES]			= new YIntStatisticEntry;			// number of matched deadlines
							properties[MISSED_DEADLINES]			= new YIntStatisticEntry;			// number of missed deadlines
							properties[DETECTED_DEADLINE_MISSES]	= new YIntStatisticEntry;			// number of discovered deadline misses
							properties[DEADLINE_TOLERANCE_STARTS]	= new YIntStatisticEntry;			// number of tolerance deadline starts
							properties[DEADLINE_TOLERANCE_MISSES]	= new YIntStatisticEntry;			// number of tolerance deadline misses
							//properties[DETECTED_TOLERANCE_MISSES]	= new YIntStatisticEntry;			// number of detected tolerance deadline misses
							properties[SYNCHRONIZATION_OFFSET]		= new YZeroYasaTimeStatisticEntry;	// synchronization offset
							properties[MIN_COMPUTATION_TIME]		= new YZeroYasaTimeStatisticEntry;	// min computation time
							properties[MAX_COMPUTATION_TIME]		= new YZeroYasaTimeStatisticEntry;	// max computation time
							properties[AVERAGE_COMPUTATION_TIME]	= new YZeroYasaTimeStatisticEntry;	// average computation time
							properties[COMPUTATION_TIME]			= new YZeroYasaTimeStatisticEntry;	// computation time of this task in the scheduling interval
							properties[UTILIZATION]					= new YPercentStatisticEntry;		// utilization in scheduling period
							properties[MIN_REACTION_TIME]			= new YZeroYasaTimeStatisticEntry;	// min. time until thread finished since activation
							properties[MAX_REACTION_TIME]			= new YZeroYasaTimeStatisticEntry;	// max. time until thread finished since activation
							properties[AVERAGE_REACTION_TIME]		= new YZeroYasaTimeStatisticEntry;	// average time until thread finished since activation
							properties[MIN_JITTER]					= new YZeroYasaTimeStatisticEntry;
							properties[MAX_JITTER]					= new YZeroYasaTimeStatisticEntry;
							properties[AVERAGE_JITTER]				= new YZeroYasaTimeStatisticEntry;
							//properties[JITTER_PERCENT]				= new YPercentStatisticEntry;
							properties[SKIPPED_THREADS]				= new YIntStatisticEntry;			// skipped threads
							properties[EMERGENCY_THREADS]			= new YIntStatisticEntry;			// how many times an emergency thread was started
							properties[LOCKED_MUTEXES]				= new YIntStatisticEntry;			// number of locked mutexes
							properties[UNLOCKED_MUTEXES]			= new YIntStatisticEntry;			// number of unlocked mutexes
							properties[RELOCKED_MUTEXES]			= new YIntStatisticEntry;			// locked a mutex twice or more ?
							properties[BLOCKED_MUTEXES]				= new YIntStatisticEntry;
							properties[MAX_PRIORITY_CEILING]		= new YPriorityStatisticEntry;		// max. priority ceiling
							properties[NUMBER_OF_PRIORITY_CHANGES]	= new YIntStatisticEntry;			//number of priority changes (How many times did a task change its priority)
							CheckProperties();
						}
	void				SetTask(YLogFileTask *t)					{	task=t;							}
	void				ResetStatistics()							{	computationtimes.clear();
																		jittertimes.clear();
																		reactiontimes.clear();
																		YLogFileStatistics::ResetStatistics();}
	int					CalculateStatistics(YEnvironmentLogFileSet &);
	virtual QStringList	GetPropertyNames() const					{	return CreatePropertyNames();	}
	static unsigned int	GetMaxProperties()							{	return MAX_PROPERTIES;			}
	static QStringList	CreatePropertyNames()
						{	QStringList	list;
							list<<MSGP_TP_ID;						// id of task
							list<<MSGP_TP_PERIOD;					// period of thread
							list<<MSGP_TP_DEADLINE;					// deadline of thread
							list<<MSGP_TP_ACTIVATIONS;				// number of activations in scheduling period
							list<<MSGP_TP_STARTS;					// number of starting periods in scheduling period
							list<<MSGP_TP_THREADS_SUSPENDED;		// how many times the thread was suspended
							list<<MSGP_TP_PREEMPTIONS;				// number of preemptions
							list<<MSGP_TP_CPU_LOSSES;				// number of cpu losts
							list<<MSGP_TP_EXCLUSIONS;				// number of exclusion (ELLF)
							list<<MSGP_TP_MATCHED_DEADLINES;		// number of matched deadlines
							list<<MSGP_TP_MISSED_DEADLINES;			// number of missed deadlines
							list<<MSGP_TP_DETECTED_DEADLINE_MISSES;	// number of discovered deadline misses
							list<<MSGP_TP_DEADLINE_TOLERANCE_STARTS;// number of tolerance deadline starts
							list<<MSGP_TP_DEADLINE_TOLERANCE_MISSES;// number of tolerance deadline misses
							//list<<MSGP_TP_DETECTED_TOLERANCE_MISSED;// number of detected tolerance deadline misses
							list<<MSGP_TP_SYNCHRONIZATION_OFFSET;	// synchronization offset
							list<<MSGP_TP_MIN_COMPUTATION_TIME;		// min computation time
							list<<MSGP_TP_MAX_COMPUTATION_TIME;		// max computation time
							list<<MSGP_TP_AVERAGE_COMPUTATION_TIME;	// average computation time
							list<<MSGP_TP_COMPUTATION_TIME,			// computation time of this task in the scheduling interval
							list<<MSGP_TP_UTILIZATION;				// utilization in scheduling period
							list<<MSGP_TP_MIN_REACTION_TIME;		// min. time until thread finished since activation
							list<<MSGP_TP_MAX_REACTION_TIME;		// max. time until thread finished since activation
							list<<MSGP_TP_AVERAGE_REACTION_TIME;	// average time until thread finished since activation
							list<<MSGP_TP_MIN_JITTER;
							list<<MSGP_TP_MAX_JITTER;
							list<<MSGP_TP_AVERAGE_JITTER;
							//list<<MSGP_TP_JITTER_PERCENT;
							list<<MSGP_TP_SKIPPED_THREADS;			// skipped threads
							list<<MSGP_TP_EMERGENCY_THREADS;		// how many times an emergency threads were started
							list<<MSGP_TP_LOCKED_MUTEXES;			// number of locked mutexes
							list<<MSGP_TP_UNLOCKED_MUTEXES;			// number of unlocked mutexes
							list<<MSGP_TP_RELOCKED_MUTEXES;			// locked a mutex twice or more ?
							list<<MSGP_TP_BLOCKED_MUTEXES;
							list<<MSGP_TP_MAX_PRIORITY_CEILING;		// max. priority ceiling
							list<<MSGP_TP_NUMBER_OF_PRIORITY_CHANGES; // number of priority changes (How many times did a task change its priority)
							return list;
						}
	void				IncActivations()							{	properties[ACTIVATIONS]->IncContent();				}
	int					GetActivations() const						{	if ( properties[ACTIVATIONS]->IsValid() )
																			return ((YIntStatisticEntry*)properties[ACTIVATIONS])->GetContent();
																		return 0;											}
	void				IncStarts()									{	properties[STARTS]->IncContent();					}
	int					GetStarts() const							{	if ( properties[STARTS]->IsValid() )
																			return ((YIntStatisticEntry*)properties[STARTS])->GetContent();
																		return 0;											}
	void				IncCPULosses()								{	properties[CPU_LOSSES]->IncContent();					}
	int					GetCPULosses() const							{	if ( properties[CPU_LOSSES]->IsValid() )
																			return ((YIntStatisticEntry*)properties[CPU_LOSSES])->GetContent();
																		return 0;											}
	void				IncPreemptions()							{	properties[PREEMPTIONS]->IncContent();				}
	int					GetPreemptions() const						{	if ( properties[PREEMPTIONS]->IsValid() )
																			return ((YIntStatisticEntry*)properties[PREEMPTIONS])->GetContent();
																		return 0;											}
	void				IncMatchedDeadlines()						{	properties[MATCHED_DEADLINES]->IncContent();		}
	void				IncMissedDeadlines()						{	properties[MISSED_DEADLINES]->IncContent();			}
	void				IncDetetectedDeadlineMisses()				{	properties[DETECTED_DEADLINE_MISSES]->IncContent();	}
	void				IncDeadlineToleranceStarts()				{	properties[DEADLINE_TOLERANCE_STARTS]->IncContent();}
	void				IncDeadlineToleranceMisses()				{	properties[DEADLINE_TOLERANCE_MISSES]->IncContent();}
	//void				IncDetectedDeadlineToleranceMisses()		{	properties[DETECTED_TOLERANCE_MISSES]->IncContent();}
	void				IncExclusions()								{	properties[EXCLUSIONS]->IncContent();				}
	int					GetExclusions() const						{	if ( properties[EXCLUSIONS]->IsValid() )
																			return ((YIntStatisticEntry*)properties[EXCLUSIONS])->GetContent();
																		return 0;											}
	void				IncSkippedThreads()							{	properties[SKIPPED_THREADS]->IncContent();			}
	int					GetSkippedThreads() const					{	if ( properties[SKIPPED_THREADS]->IsValid() )
																			return ((YIntStatisticEntry*)properties[SKIPPED_THREADS])->GetContent();
																		return 0;											}
	void				IncSuspendedThreads()						{	properties[THREADS_SUSPENDED]->IncContent();		}
	int					GetSuspendedThreads() const					{	if ( properties[THREADS_SUSPENDED]->IsValid() )
																			return ((YIntStatisticEntry*)properties[THREADS_SUSPENDED])->GetContent();
																		return 0;											}
	void				IncEmergencyThreads()						{	properties[EMERGENCY_THREADS]->IncContent();		}
	int					GetEmergencyThreads() const					{	if ( properties[EMERGENCY_THREADS]->IsValid() )
																			return ((YIntStatisticEntry*)properties[EMERGENCY_THREADS])->GetContent();
																		return 0;											}
	void				IncLockedMutexes()							{	properties[LOCKED_MUTEXES]->IncContent();			}
	void				IncUnlockedMutexes()						{	properties[UNLOCKED_MUTEXES]->IncContent();			}
	void				IncRelockedMutexes()						{	properties[RELOCKED_MUTEXES]->IncContent();			}
	void				IncBlockedMutexes()							{	properties[BLOCKED_MUTEXES]->IncContent();			}
	void				SetPeriod(YASA_TIME t)						{	((YInfinityYasaTimeStatisticEntry*)properties[PERIOD])->SetContent(t);			}
	void				SetDeadline(YASA_TIME t)					{	((YInfinityYasaTimeStatisticEntry*)properties[DEADLINE])->SetContent(t);		}
	YASA_TIME			GetPeriod() const							{	return ((YInfinityYasaTimeStatisticEntry*)properties[PERIOD])->GetContent();	}
	YASA_TIME			GetDeadline() const							{	return ((YInfinityYasaTimeStatisticEntry*)properties[DEADLINE])->GetContent();	}
	void				SetMaxPriorityCeiling(int p)				{	if ( ! (properties[MAX_PRIORITY_CEILING])->IsValid() || ((YIntStatisticEntry*)properties[MAX_PRIORITY_CEILING])->GetContent() < p )
																			((YIntStatisticEntry*)properties[MAX_PRIORITY_CEILING])->SetContent(p);		}
	void				SetSynchronizationOffset(YASA_TIME t)		{	((YYasaTimeStatisticEntry*)properties[SYNCHRONIZATION_OFFSET])->SetContent(t);	}
	void				InvalidSynchronizationOffset()				{	((YYasaTimeStatisticEntry*)properties[SYNCHRONIZATION_OFFSET])->Invalid();		}
	void				InvalidJitter()								{	((YYasaTimeStatisticEntry*)properties[MIN_JITTER])->Invalid();
																		((YYasaTimeStatisticEntry*)properties[MAX_JITTER])->Invalid();
																		((YYasaTimeStatisticEntry*)properties[AVERAGE_JITTER])->Invalid();				}
	void				InvalidPeriod()								{	((YYasaTimeStatisticEntry*)properties[PERIOD])->Invalid();						}
	void				InvalidDeadline()							{	((YYasaTimeStatisticEntry*)properties[DEADLINE])->Invalid();					}
	void				InvalidPreemptions()						{	((YIntStatisticEntry*)properties[PREEMPTIONS])->Invalid();						}
	void				InvalidCPULosts()							{	((YIntStatisticEntry*)properties[CPU_LOSSES])->Invalid();						}
	void				InvalidMatchedDeadlines()					{	((YIntStatisticEntry*)properties[MATCHED_DEADLINES])->Invalid();				}
	void				InvalidMissedDeadlines()					{	((YIntStatisticEntry*)properties[MISSED_DEADLINES])->Invalid();					}
	void				InvalidDetectedDeadlineMissed()				{	((YIntStatisticEntry*)properties[DETECTED_DEADLINE_MISSES])->Invalid();			}
	void				InvalidDeadlineToleranceStarts()			{	((YIntStatisticEntry*)properties[DEADLINE_TOLERANCE_STARTS])->Invalid();		}
	void				InvalidDeadlineToleranceMisses()			{	((YIntStatisticEntry*)properties[DEADLINE_TOLERANCE_MISSES])->Invalid();		}
	void				InvalidReactionTime()						{	((YYasaTimeStatisticEntry*)properties[MIN_REACTION_TIME])->Invalid();
																		((YYasaTimeStatisticEntry*)properties[MAX_REACTION_TIME])->Invalid();
																		((YYasaTimeStatisticEntry*)properties[AVERAGE_REACTION_TIME])->Invalid();		}
	void				InvalidExclusions()							{	((YIntStatisticEntry*)properties[EXCLUSIONS])->Invalid();						}
	void				InvalidSkipped()							{	((YIntStatisticEntry*)properties[SKIPPED_THREADS])->Invalid();					}
	void				InvalidSuspended()							{	((YIntStatisticEntry*)properties[THREADS_SUSPENDED])->Invalid();				}
	void				InvalidEmergencyThreads()					{	((YIntStatisticEntry*)properties[EMERGENCY_THREADS])->Invalid();				}
	void				InvalidLockedMutexes()						{	((YIntStatisticEntry*)properties[LOCKED_MUTEXES])->Invalid();					}
	void				InvalidUnlockedMutexes()					{	((YIntStatisticEntry*)properties[UNLOCKED_MUTEXES])->Invalid();					}
	void				InvalidMutexesRelocked()					{	((YIntStatisticEntry*)properties[RELOCKED_MUTEXES])->Invalid();					}
	void				InvalidBlockedByMutexes()					{	((YIntStatisticEntry*)properties[BLOCKED_MUTEXES])->Invalid();					}
	void				InvalidMaxPriorityCeiling()					{	((YPriorityStatisticEntry*)properties[MAX_PRIORITY_CEILING])->Invalid();		}
	void				InvalidNumberOfPriorityChanges()			{	((YIntStatisticEntry*)properties[NUMBER_OF_PRIORITY_CHANGES])->Invalid();		}
	YASA_TIME			GetComputationTime() const					{	return ((YZeroYasaTimeStatisticEntry*)properties[COMPUTATION_TIME])->GetContent();}
	void				AddComputationTime(YASA_TIME t)				{	computationtimes.append( new YASA_TIME(t) );									}
	void				AddJitterTime(YASA_TIME t)					{	jittertimes.append( new YASA_TIME(t) );											}
	void				AddReactionTime(YASA_TIME t)				{	reactiontimes.append( new YASA_TIME(t) );										}
	void				IncNumberOfPriorityChanges()				{	properties[NUMBER_OF_PRIORITY_CHANGES]->IncContent();							}
};




//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This class contains the global statistic entries of one project
class YMutexLogFileStatistics : public YLogFileStatistics
{	enum MUTEX_PROPERTIES {	LOCKED=0,					// number of lock
							RELOCKED,					// number of allready locked mutexes
							UNLOCKED,					// number of unlocked mutexes
							BLOCKED,					// threads blocked
							MAX_PRIORITY_CEILING,		// max. priority ceiling of this mutex
							NUMBER_OF_PRIORITY_CHANGES,	// number of priority changes (How many times did a task change its priority)
							MIN_LOCKING_TIME,			// min. locking time of mutex
							MAX_LOCKING_TIME,			// max. locking time of mutex
							AVERAGE_LOCKING_TIME,		// average locking time of mutex
							MAX_PROPERTIES
						};
	YLogFileMutex *		mutex;
	QList<YASA_TIME>	lockingtimes;
public:					YMutexLogFileStatistics() : YLogFileStatistics(MAX_PROPERTIES), mutex(0)
						{
							properties[LOCKED]						= new YIntStatisticEntry;
							properties[RELOCKED]					= new YIntStatisticEntry;
							properties[UNLOCKED]					= new YIntStatisticEntry;
							properties[BLOCKED]						= new YIntStatisticEntry;
							properties[MAX_PRIORITY_CEILING]		= new YPriorityStatisticEntry;
							properties[NUMBER_OF_PRIORITY_CHANGES]	= new YIntStatisticEntry;
							properties[MIN_LOCKING_TIME]			= new YZeroYasaTimeStatisticEntry;
							properties[MAX_LOCKING_TIME]			= new YZeroYasaTimeStatisticEntry;
							properties[AVERAGE_LOCKING_TIME]		= new YZeroYasaTimeStatisticEntry;
							CheckProperties();
						}
	void				SetMutex(YLogFileMutex *m)					{	mutex=m;							}
	void				IncMutexLocked()							{	properties[LOCKED]->IncContent();	}
	void				IncMutexRelocked()							{	properties[RELOCKED]->IncContent();	}
	void				IncMutexUnlocked()							{	properties[UNLOCKED]->IncContent();	}
	void				IncMutexBlocked()							{	properties[BLOCKED]->IncContent();	}
	void				SetMaxPriorityCeiling(int p)				{	if ( ! (properties[MAX_PRIORITY_CEILING])->IsValid() || ((YIntStatisticEntry*)properties[MAX_PRIORITY_CEILING])->GetContent() < p )
																			((YIntStatisticEntry*)properties[MAX_PRIORITY_CEILING])->SetContent(p);		}
	void				IncNumberOfPriorityChanges()				{	properties[NUMBER_OF_PRIORITY_CHANGES]->IncContent();	}
	int					CalculateStatistics(YEnvironmentLogFileSet &);
	virtual QStringList	GetPropertyNames() const					{	return CreatePropertyNames();	}
	static unsigned int	GetMaxProperties()							{	return MAX_PROPERTIES;			}
	void				AddLockingTime(YASA_TIME t)					{	lockingtimes.append( new YASA_TIME(t) );	}
	static QStringList	CreatePropertyNames()
						{	QStringList	list;
							list<<MSGP_MP_LOCKED;
							list<<MSGP_MP_RELOCKED;
							list<<MSGP_MP_UNLOCKED;
							list<<MSGP_MP_BLOCKED;
							list<<MSGP_MP_MAX_PRIORITY_CEILING;
							list<<MSGP_MP_NUMBER_OF_PRIORITY_CHANGES;
							list<<MSGP_MP_MIN_LOCKING_TIME;
							list<<MSGP_MP_MAX_LOCKING_TIME;
							list<<MSGP_MP_AVERAGE_LOCKING_TIME;
							return list;
						}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This class contains the global statistic entries of one project
class YCPULogFileStatistics : public YLogFileStatistics
{	enum CPU_PROPERTIES	{	SCHEDULER_NAME=0,					// name of scheduler
							SCHEDULER_VERSION,					// version of scheduler
							NUMBER_OF_CPU,						// number of CPU
							EXECUTION_START,					// time of start
							SCHEDULED_EXECUTION_END,			// scheduled execution end
							EXECUTION_END,						// time of end
							EXECUTION_DURATION,					// duration of scheduling
							SCHEDULER_TICK,						// scheduling tick
							CALC_PREEMPTION_TIME,				// calc preemption time enabled ?
							SIZE_OF_LOGBUFFER,					// usage of logfile buffer (bytes)
							USAGE_OF_LOGBUFFER,					// usage of logfile buffer (bytes)
							USAGE_OF_LOGBUFFER_PERCENT,			// usage of logfile buffer (percent)
							NUMBER_OF_TASKS,					// number of tasks
							TASK_SWITCHES,						// task switches
							TASK_CPU_SWITCHES,					// switches of task's between cpus (How many times did any task change between this and other cpu's)
							MATCHED_DEADLINES,					// matched deadlines
							MISSED_DEADLINES,					// missed deadlines
							DETECTED_DEADLINE_MISSES,			// detected deadline misses
							MISSED_TOLERANCE_DEADLINES,			// missed tolerance deadlines
							UTILIZATION,						// Utilization of cpu
							UTILIZATION_PERCENT,				// Utilization of cpu (%)
							IDLE,								// idle time
							IDLE_PERCENT,						// idle time (percent)
							SCHEDULER_CALLS,					// number of scheduler calls
							TIME_OF_SCHEDULING,					// time of scheduling
							TIME_OF_SCHEDULING_PERCENT,			// time of scheduling(percent)
							MUTEX_LOCKED,						// locked mutexes on this cpu
							MUTEX_RELOCKED,						// relocked mutexes on this cpu
							MUTEX_UNLOCKED,						// unlocked mutexes on this cpu
							MUTEX_BLOCKED,						// blocked mutexes on this cpu
							MUTEX_MAX_PRIORITY_CEILING,			// max. priority changed through mutexes
							MUTEX_NUMBER_OF_PRIORITY_CHANGES,	// number priority changed on this cpu
							MAX_PROPERTIES
						};
public:					YCPULogFileStatistics() : YLogFileStatistics(MAX_PROPERTIES)	//, cpulogfile(0)
						{
							properties[SCHEDULER_NAME]				= new YStringStatisticEntry;		// name of scheduler
							properties[SCHEDULER_VERSION]			= new YVersionStatisticEntry;		// version of scheduler
							properties[NUMBER_OF_CPU]				= new YIntStatisticEntry;			// number of CPU
							properties[EXECUTION_START]				= new YZeroYasaTimeStatisticEntry;	// time of start
							properties[SCHEDULED_EXECUTION_END]		= new YInfinityYasaTimeStatisticEntry;	// scheduled execution time
							properties[EXECUTION_END]				= new YInfinityYasaTimeStatisticEntry;	// time of end
							properties[EXECUTION_DURATION]			= new YInfinityYasaTimeStatisticEntry;	// duration
							properties[SCHEDULER_TICK]				= new YZeroYasaTimeStatisticEntry;	// scheduling tick
							properties[CALC_PREEMPTION_TIME]		= new YBoolStatisticEntry;			// calc preemption time enabled ?
							properties[SIZE_OF_LOGBUFFER]			= new YIntStatisticEntry;			// usage of logfile buffer (bytes)
							properties[USAGE_OF_LOGBUFFER]			= new YIntStatisticEntry;			// usage of logfile buffer (bytes)
							properties[USAGE_OF_LOGBUFFER_PERCENT]	= new YPercentStatisticEntry;		// usage of logfile buffer (percent)
							properties[NUMBER_OF_TASKS]				= new YIntStatisticEntry;			// number of tasks
							properties[TASK_SWITCHES]				= new YIntStatisticEntry;			// task switches
							properties[TASK_CPU_SWITCHES]			= new YIntStatisticEntry;			// switches of task's between cpus (How many times did any task change between this and other cpu's)
							properties[MATCHED_DEADLINES]			= new YIntStatisticEntry;			// matched deadlines
							properties[MISSED_DEADLINES]			= new YIntStatisticEntry;			// missed deadlines
							properties[DETECTED_DEADLINE_MISSES]	= new YIntStatisticEntry;			// detected deadlines misses
							properties[MISSED_TOLERANCE_DEADLINES]	= new YIntStatisticEntry;			// missed tolerance deadlines
							properties[UTILIZATION]					= new YZeroYasaTimeStatisticEntry;	// utilization
							properties[UTILIZATION_PERCENT]			= new YPercentStatisticEntry;		// utilization (percent)
							properties[IDLE]						= new YZeroYasaTimeStatisticEntry;	// idle time
							properties[IDLE_PERCENT]				= new YPercentStatisticEntry;		// idle time (percent)
							properties[SCHEDULER_CALLS]				= new YIntStatisticEntry;			// number of scheduler calls
							properties[TIME_OF_SCHEDULING]			= new YZeroYasaTimeStatisticEntry;	// time of scheduling
							properties[TIME_OF_SCHEDULING_PERCENT]	= new YPercentStatisticEntry;		// time of scheduling(percent)
							properties[MUTEX_LOCKED]				= new YIntStatisticEntry;
							properties[MUTEX_RELOCKED]				= new YIntStatisticEntry;
							properties[MUTEX_UNLOCKED]				= new YIntStatisticEntry;
							properties[MUTEX_BLOCKED]				= new YIntStatisticEntry;
							properties[MUTEX_MAX_PRIORITY_CEILING]	= new YPriorityStatisticEntry;
							properties[MUTEX_NUMBER_OF_PRIORITY_CHANGES]= new YIntStatisticEntry;
							CheckProperties();
						}
	int					CalculateStatistics(YCPULogFile &);
	virtual QStringList	GetPropertyNames() const					{	return CreatePropertyNames();	}
	static unsigned int	GetMaxProperties()							{	return MAX_PROPERTIES;			}
	static QStringList	CreatePropertyNames()
						{	QStringList	list;
							list<<MSGP_CP_SCHEDULER_NAME;
							list<<MSGP_CP_SCHEDULER_VERSION;
							list<<MSGP_CP_NUMBER_OF_CPU;
							list<<MSGP_CP_EXECUTION_START;
							list<<MSGP_CP_SCHEDULES_EXECUTION_END;
							list<<MSGP_CP_EXECUTION_END;
							list<<MSGP_CP_EXECUTION_DURATION;
							list<<MSGP_CP_SCHEDULER_TICK;
							list<<MSGP_CP_CALC_PREEMPTION_TIME;
							list<<MSGP_CP_SIZE_OF_LOGBUFFER;
							list<<MSGP_CP_USAGE_OF_LOGBUFFER;
							list<<MSGP_CP_USAGE_OF_LOGBUFFER_PERCENT;
							list<<MSGP_CP_NUMBER_OF_TASKS;
							list<<MSGP_CP_TASK_SWITCHES;
							list<<MSGP_CP_TASK_CPU_SWITCHES;
							list<<MSGP_CP_MATCHED_DEADLINES;
							list<<MSGP_CP_MISSED_DEADLINES;
							list<<MSGP_CP_DETECTED_DEADLINE_MISSES;
							list<<MSGP_CP_MISSED_TOLERANCE_DEADLINES;
							list<<MSGP_CP_UTILIZATION;
							list<<MSGP_CP_UTILIZATION_PERCENT;
							list<<MSGP_CP_IDLE;
							list<<MSGP_CP_IDLE_PERCENT;
							list<<MSGP_CP_SCHEDULER_CALLS;
							list<<MSGP_CP_TIME_OF_SCHEDULING;
							list<<MSGP_CP_TIME_OF_SCHEDULING_PERCENT;
							list<<MSGP_CP_LOCKED;
							list<<MSGP_CP_RELOCKED;
							list<<MSGP_CP_UNLOCKED;
							list<<MSGP_CP_BLOCKED;
							list<<MSGP_CP_MAX_PRIORITY_CEILING;
							list<<MSGP_CP_NUMBER_OF_PRIORITY_CHANGES;
							return list;
						}
	void				SetSchedulerVersion(double v)				{	((YVersionStatisticEntry*)properties[SCHEDULER_VERSION])->SetContent(v);		}
	void				SetSchedulerName(const QString &name)		{	((YStringStatisticEntry*)properties[SCHEDULER_NAME])->SetContent(name);			}
	void				SetNumberOfCPU(int cpu)						{	((YIntStatisticEntry*)properties[NUMBER_OF_CPU])->SetContent(cpu);				}
	int					GetNumberOfCPU() const						{	return ((YIntStatisticEntry*)properties[NUMBER_OF_CPU])->GetContent();			}
	void				SetExecutionStart(YASA_TIME time)			{	((YYasaTimeStatisticEntry*)properties[EXECUTION_START])->SetContent(time);		}
	void				SetScheduledExecutionEnd(YASA_TIME time)	{	((YYasaTimeStatisticEntry*)properties[SCHEDULED_EXECUTION_END])->SetContent(time);		}
	YASA_TIME			GetScheduledExecutionEnd() const			{	return ((YYasaTimeStatisticEntry*)properties[SCHEDULED_EXECUTION_END])->GetContent();	}
	void				SetExecutionEnd(YASA_TIME time)				{	((YYasaTimeStatisticEntry*)properties[EXECUTION_END])->SetContent(time);		}
	YASA_TIME			GetExecutionEnd() const						{	return ((YYasaTimeStatisticEntry*)properties[EXECUTION_END])->GetContent();		}
	void				SetExecutionDuration(YASA_TIME time)		{	((YYasaTimeStatisticEntry*)properties[EXECUTION_DURATION])->SetContent(time);	}
	YASA_TIME			GetExecutionDuration() const				{	return ((YYasaTimeStatisticEntry*)properties[EXECUTION_DURATION])->GetContent();}
	void				SetSchedulerTick(YASA_TIME time)			{	((YYasaTimeStatisticEntry*)properties[SCHEDULER_TICK])->SetContent(time);		}
	YASA_TIME			GetSchedulerTick() const					{	return ((YYasaTimeStatisticEntry*)properties[SCHEDULER_TICK])->GetContent();	}
	void				SetCalcPreemptionTime(bool b)				{	((YBoolStatisticEntry*)properties[CALC_PREEMPTION_TIME])->SetContent(b);		}
	void				SetLogBufferSize(int s)						{	((YIntStatisticEntry*)properties[SIZE_OF_LOGBUFFER])->SetContent(s);			}
	int					GetLogBufferSize() const					{	return ((YIntStatisticEntry*)properties[SIZE_OF_LOGBUFFER])->GetContent();		}
	int					GetUsedLogBufferSize() const				{	return ((YIntStatisticEntry*)properties[USAGE_OF_LOGBUFFER])->GetContent();		}
	void				SetUsedLogBufferSize(int s)					{	((YIntStatisticEntry*)properties[USAGE_OF_LOGBUFFER])->SetContent(s);
																		if ( properties[SIZE_OF_LOGBUFFER]->IsValid() )
																		{
																			((YPercentStatisticEntry*)properties[USAGE_OF_LOGBUFFER_PERCENT])->CalcContent( ((YIntStatisticEntry*)properties[SIZE_OF_LOGBUFFER])->GetContent(), s );
																		}
																	}
	void				SetNumberOfTasks(int n)						{	((YIntStatisticEntry*)properties[NUMBER_OF_TASKS])->SetContent(n);				}
	//int					GetNumberOfTasks() const				{	return ((YIntStatisticEntry*)properties[NUMBER_OF_TASKS])->GetContent();		}
	void				SetTaskSwitches(int sw)						{	((YIntStatisticEntry*)properties[TASK_SWITCHES])->SetContent(sw);				}
	int					GetTaskSwitches()							{	return ((YIntStatisticEntry*)properties[TASK_SWITCHES])->GetContent();			}
	void				SetMatchedDeadlines(int md)					{	((YIntStatisticEntry*)properties[MATCHED_DEADLINES])->SetContent(md);			}
	int					GetMatchedDeadlines() const					{	return ((YIntStatisticEntry*)properties[MATCHED_DEADLINES])->GetContent();		}
	void				SetMissedDeadlines(int md)					{	((YIntStatisticEntry*)properties[MISSED_DEADLINES])->SetContent(md);			}
	int					GetMissedDeadlines() const					{	return ((YIntStatisticEntry*)properties[MISSED_DEADLINES])->GetContent();		}
	void				SetDetectedDeadlineMisses(int ddm)			{	((YIntStatisticEntry*)properties[DETECTED_DEADLINE_MISSES])->SetContent(ddm);	}
	int					GetDetectedDeadlineMisses() const			{	return ((YIntStatisticEntry*)properties[DETECTED_DEADLINE_MISSES])->GetContent(); }
	void				SetDeadlineToleranceMisses(int md)			{	((YIntStatisticEntry*)properties[MISSED_TOLERANCE_DEADLINES])->SetContent(md);	}
	int					GetDeadlineToleranceMisses() const			{	return ((YIntStatisticEntry*)properties[MISSED_TOLERANCE_DEADLINES])->GetContent();	}
	void				SetTaskCPUSwitches(int sw)					{	((YIntStatisticEntry*)properties[TASK_CPU_SWITCHES])->SetContent(sw);			}
	int					GetTaskCPUSwitches() const					{	return ((YIntStatisticEntry*)properties[TASK_CPU_SWITCHES])->GetContent();		}
	YASA_TIME			GetUtilization() const						{	return ((YYasaTimeStatisticEntry*)properties[UTILIZATION])->GetContent();		}
	PERCENTTYPE			GetUtilizationPercent() const				{	return ((YPercentStatisticEntry*)properties[UTILIZATION_PERCENT])->GetContent();}
	void				SetUtilization(YASA_TIME time)				{	((YYasaTimeStatisticEntry*)properties[UTILIZATION])->SetContent(time);
																		if ( properties[EXECUTION_DURATION]->IsValid() )
																		{
																			((YPercentStatisticEntry*)properties[UTILIZATION_PERCENT])->CalcContent( ((YYasaTimeStatisticEntry*)properties[EXECUTION_DURATION])->GetContent(), time );
																		}
																	}
	YASA_TIME			GetIdleTime() const							{	return ((YYasaTimeStatisticEntry*)properties[IDLE])->GetContent();				}
	void				SetIdleTime(YASA_TIME time)					{	((YYasaTimeStatisticEntry*)properties[IDLE])->SetContent(time);
																		if ( properties[EXECUTION_DURATION]->IsValid() )
																		{
																			((YPercentStatisticEntry*)properties[IDLE_PERCENT])->CalcContent( ((YYasaTimeStatisticEntry*)properties[EXECUTION_DURATION])->GetContent(), time );
																		}
																	}
	void				SetSchedulerCalls(int sc)					{	((YIntStatisticEntry*)properties[SCHEDULER_CALLS])->SetContent(sc);				}
	int					GetSchedulerCalls() const					{	return ((YIntStatisticEntry*)properties[SCHEDULER_CALLS])->GetContent();		}
	YASA_TIME			GetTimeOfScheduling() const					{	return ((YYasaTimeStatisticEntry*)properties[TIME_OF_SCHEDULING])->GetContent();}
	void				SetTimeOfScheduling(YASA_TIME time)			{	((YYasaTimeStatisticEntry*)properties[TIME_OF_SCHEDULING])->SetContent(time);
																		if ( properties[EXECUTION_DURATION]->IsValid() )
																		{
																			((YPercentStatisticEntry*)properties[TIME_OF_SCHEDULING_PERCENT])->CalcContent( ((YYasaTimeStatisticEntry*)properties[EXECUTION_DURATION])->GetContent(), time );
																		}
																	}
	void				IncMutexLocked()							{	properties[MUTEX_LOCKED]->IncContent();											}
	int					GetMutexLocked() const						{	return ((YIntStatisticEntry*)properties[MUTEX_LOCKED])->GetContent();			}
	void				IncMutexRelocked()							{	properties[MUTEX_RELOCKED]->IncContent();										}
	int					GetMutexRelocked() const					{	return ((YIntStatisticEntry*)properties[MUTEX_RELOCKED])->GetContent();			}
	void				IncMutexUnlocked()							{	properties[MUTEX_UNLOCKED]->IncContent();										}
	int					GetMutexUnlocked() const					{	return ((YIntStatisticEntry*)properties[MUTEX_UNLOCKED])->GetContent();			}
	void				IncMutexBlocked()							{	properties[MUTEX_BLOCKED]->IncContent();										}
	int					GetMutexBlocked() const						{	return ((YIntStatisticEntry*)properties[MUTEX_BLOCKED])->GetContent();			}
	void				SetMaxPriorityCeiling(int p)				{	if ( ! properties[MUTEX_MAX_PRIORITY_CEILING]->IsValid() || ((YPriorityStatisticEntry*)properties[MUTEX_MAX_PRIORITY_CEILING])->GetContent() < p )
																			((YPriorityStatisticEntry*)properties[MUTEX_MAX_PRIORITY_CEILING])->SetContent(p);		}
	int					GetMaxPriorityCeiling() const				{	return ((YPriorityStatisticEntry*)properties[MUTEX_MAX_PRIORITY_CEILING])->GetContent();	}
	void				IncNumberOfPriorityChanges()				{	properties[MUTEX_NUMBER_OF_PRIORITY_CHANGES]->IncContent();								}
	int					GetNumberOfPriorityChanges() const			{	return ((YIntStatisticEntry*)properties[MUTEX_NUMBER_OF_PRIORITY_CHANGES])->GetContent();	}
};



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This class contains the global statistic entries of one environment
class YEnvironmentLogFileStatistics : public YLogFileStatistics
{
	enum ENVIRONMENT_PROPERTIES	{
							EXECUTIVE_NAME=0,			// name of executive
							EXECUTIVE_VERSION,			// version of executive
							LF_YASA_VERSION,			// version of yasa used in logfile
							NUMBER_OF_CPUS,				// number of CPU
							//UTILIZATION_OF_CPUS,		// utilization of all cpus
							UTILIZATION_OF_CPUS_PERCENT,// utilization of all cpus (%)
							SCHEDULER_CALLS,			// number of scheduler calls
							TASK_SWITCHES,				// task switches
							TASK_CPU_SWITCHES,			// switches of task's between cpus (How many times did any task change between this and other cpu's)
							ACTIVATIONS,				// number of activations in scheduling period
							STARTS,						// number of starting periods in scheduling period
							THREADS_SUSPENDED,			// how many times the thread was suspended
							PREEMPTIONS,				// number of preemptions
							CPU_LOSSES,					// number of cpu losts
							EXCLUSIONS,					// number of exclusion (ELLF)
							SKIPPED_THREADS,			// skipped threads
							EMERGENCY_THREADS,			// how many times an emergency thread was started
							MATCHED_DEADLINES,			// matched deadlines
							MISSED_DEADLINES,			// missed deadlines
							DETECTED_DEADLINE_MISSES,	// detected deadline misses
							MISSED_TOLERANCE_DEADLINES,	// missed tolerance deadlines
							SIZE_OF_LOGBUFFER,			// usage of logfile buffer (bytes)
							USAGE_OF_LOGBUFFER,			// usage of logfile buffer (bytes)
							USAGE_OF_LOGBUFFER_PERCENT,	// usage of logfile buffer (percent)
							IDLE,						// idle time
							IDLE_PERCENT,				// idle time (percent)
							TIME_OF_SCHEDULING,			// time of scheduling
							TIME_OF_SCHEDULING_PERCENT,	// time of scheduling(percent)
							MUTEX_LOCKED,				// locked mutexes on this cpu
							MUTEX_RELOCKED,				// relocked mutexes on this cpu
							MUTEX_UNLOCKED,				// unlocked mutexes on this cpu
							MUTEX_BLOCKED,				// blocked mutexes on this cpu
							MUTEX_MAX_PRIORITY_CEILING,	// max. priority changed through mutexes
							MUTEX_NUMBER_OF_PRIORITY_CHANGES,	// number priority changed on this cpu
							MAX_PROPERTIES
						};
public:					YEnvironmentLogFileStatistics() : YLogFileStatistics(MAX_PROPERTIES)
						{
							properties[EXECUTIVE_NAME]				= new YStringStatisticEntry;		// name of executive
							properties[EXECUTIVE_VERSION]			= new YVersionStatisticEntry;		// version of executive
							properties[LF_YASA_VERSION]				= new YVersionStatisticEntry;		// version of yasa
							properties[NUMBER_OF_CPUS]				= new YIntStatisticEntry;			// cpus
							//properties[UTILIZATION_OF_CPUS]		= new YIntStatisticEntry;			// utilization
							properties[UTILIZATION_OF_CPUS_PERCENT]	= new YPercentStatisticEntry;		// utilization %
							properties[SCHEDULER_CALLS]				= new YIntStatisticEntry;			// schedulercalls;
							properties[TASK_SWITCHES]				= new YIntStatisticEntry;			// taskswitches;
							properties[TASK_CPU_SWITCHES]			= new YIntStatisticEntry;			// taskcpuswitches;
							properties[ACTIVATIONS]					= new YIntStatisticEntry;			// number of activations in scheduling period
							properties[STARTS]						= new YIntStatisticEntry;			// number of starting periods in scheduling period
							properties[THREADS_SUSPENDED]			= new YIntStatisticEntry;			// how many times the thread was suspended
							properties[PREEMPTIONS]					= new YIntStatisticEntry;			// number of preemptions
							properties[CPU_LOSSES]					= new YIntStatisticEntry;			// number of cpu losts
							properties[EXCLUSIONS]					= new YIntStatisticEntry;			// number of exclusion (ELLF)
							properties[SKIPPED_THREADS]				= new YIntStatisticEntry;			// skipped threads
							properties[EMERGENCY_THREADS]			= new YIntStatisticEntry;			// how many times an emergency thread was started
							properties[MATCHED_DEADLINES]			= new YIntStatisticEntry;			// matched deadlines
							properties[MISSED_DEADLINES]			= new YIntStatisticEntry;			// missed deadlines
							properties[DETECTED_DEADLINE_MISSES]	= new YIntStatisticEntry;			// detected deadlines misses
							properties[MISSED_TOLERANCE_DEADLINES]	= new YIntStatisticEntry;			// missed tolerance deadlines
							properties[SIZE_OF_LOGBUFFER]			= new YIntStatisticEntry;			// logbuffersize;
							properties[USAGE_OF_LOGBUFFER]			= new YIntStatisticEntry;			// usedlogbuffer;
							properties[USAGE_OF_LOGBUFFER_PERCENT]	= new YPercentStatisticEntry;		// usedlogbufferpercent;
							properties[IDLE]						= new YZeroYasaTimeStatisticEntry;	// idletime;
							properties[IDLE_PERCENT]				= new YPercentStatisticEntry;		// idletimepercent;
							properties[TIME_OF_SCHEDULING]			= new YZeroYasaTimeStatisticEntry;	// schedulingtime;
							properties[TIME_OF_SCHEDULING_PERCENT]	= new YPercentStatisticEntry;		// schedulingpercent;
							properties[MUTEX_LOCKED]				= new YIntStatisticEntry;
							properties[MUTEX_RELOCKED]				= new YIntStatisticEntry;
							properties[MUTEX_UNLOCKED]				= new YIntStatisticEntry;
							properties[MUTEX_BLOCKED]				= new YIntStatisticEntry;
							properties[MUTEX_MAX_PRIORITY_CEILING]	= new YPriorityStatisticEntry;
							properties[MUTEX_NUMBER_OF_PRIORITY_CHANGES]= new YIntStatisticEntry;
							CheckProperties();
						}
	void				SetYasaVersion(double v)					{	((YVersionStatisticEntry*)properties[LF_YASA_VERSION])->SetContent(v);		}
	void				SetExecutiveVersion(double v)				{	((YVersionStatisticEntry*)properties[EXECUTIVE_VERSION])->SetContent(v);	}
	void				SetExecutiveName(const QString &name)		{	((YStringStatisticEntry*)properties[EXECUTIVE_NAME])->SetContent(name);		}
	const QString &		GetExecutiveName() const					{	return ((YStringStatisticEntry*)properties[EXECUTIVE_NAME])->GetContent();	}
	int					CalculateStatistics(YEnvironmentLogFileSet &);
	virtual QStringList	GetPropertyNames() const					{	return CreatePropertyNames();	}
	static unsigned int	GetMaxProperties()							{	return MAX_PROPERTIES;			}
	static QStringList	CreatePropertyNames()
						{	QStringList	list;
							list<<MSGP_EP_EXECUTIVE_NAME;
							list<<MSGP_EP_EXECUTIVE_VERSION;
							list<<MSGP_EP_YASA_VERSION;
							list<<MSGP_EP_NUMBER_OF_CPUS;
							//list<<MSGP_EP_UTILIZATION_OF_CPUS;
							list<<MSGP_EP_UTILIZATION_OF_CPUS_PERCENT;
							list<<MSGP_EP_SCHEDULER_CALLS;
							list<<MSGP_EP_TASK_SWITCHES;
							list<<MSGP_EP_TASK_CPU_SWITCHES;
							list<<MSGP_EP_ACTIVATIONS;				// number of activations in scheduling period
							list<<MSGP_EP_STARTS;					// number of starting periods in scheduling period
							list<<MSGP_EP_THREADS_SUSPENDED;		// how many times the thread was suspended
							list<<MSGP_EP_PREEMPTIONS;				// number of preemptions
							list<<MSGP_EP_CPU_LOSSES;					// number of cpu losts
							list<<MSGP_EP_EXCLUSIONS;				// number of exclusion (ELLF)
							list<<MSGP_EP_SKIPPED_THREADS;			// skipped threads
							list<<MSGP_EP_EMERGENCY_THREADS;		// how many times an emergency threads were started
							list<<MSGP_EP_MATCHED_DEADLINES;
							list<<MSGP_EP_MISSED_DEADLINES;
							list<<MSGP_EP_DETECTED_DEADLINE_MISSES;
							list<<MSGP_EP_MISSED_TOLERANCE_DEADLINES;
							list<<MSGP_EP_SIZE_OF_LOGBUFFER;
							list<<MSGP_EP_USAGE_OF_LOGBUFFER;
							list<<MSGP_EP_USAGE_OF_LOGBUFFER_PERCENT;
							list<<MSGP_EP_IDLE;
							list<<MSGP_EP_IDLE_PERCENT;
							list<<MSGP_EP_TIME_OF_SCHEDULING;
							list<<MSGP_EP_TIME_OF_SCHEDULING_PERCENT;
							list<<MSGP_EP_LOCKED;
							list<<MSGP_EP_RELOCKED;
							list<<MSGP_EP_UNLOCKED;
							list<<MSGP_EP_BLOCKED;
							list<<MSGP_EP_MAX_PRIORITY_CEILING;
							list<<MSGP_EP_NUMBER_OF_PRIORITY_CHANGES;
							return list;
						}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif																// ifndef YASAGUI_LOGFILESTATISTICS_PARSER_INCLUDE
